#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#include "../include/color.h"

using namespace VMI;

Color *VMI::initColors(int numColors) {
    Color *colors;

    colors = (Color *)malloc(sizeof(Color) * (numColors + 1));
		/* The 0 position is reserved for the BACKGROUND, thus color of triangle 0
       is at 1 position and so on */

    if (colors == NULL) {
        fprintf(stderr, "Error allocating memory\n");
        exit(1);
    }

    /* Fill the color buffer with the BACKGROUND color */
    memset(colors, BACKGROUND, sizeof(Color) * (numColors + 1));

    return colors;
}

void VMI::fillAllColors(Color *colors, int numColors, int begin, int end, GLubyte color) {
    int i;

    if (end > numColors) return;

    for (i=begin; i<end; i++) {

        colors[i].r = color;
        colors[i].g = color;
        colors[i].b = color;
        colors[i].a = color;
    }

}

void VMI::setColors(Color *colors, int numPasses, int begin, int end) {
    int i, step = numPasses * MAX_NUM_COLORS;
    GLubyte r = 1,
            g = 1,
            b = 1,
            a = 1;
    
    for (i = begin; i < end; i++) {
        if (i < (255 + step)) {
            colors[i].r = r;
            r++;
            colors[i].g = 0;
            colors[i].b = 0;
            colors[i].a = 0;

        } 
        if ((i >= (255 + step)) && (i < (510 + step))) {
            colors[i].r = 0;
            colors[i].g = g;
            g++;
            colors[i].b = 0;
            colors[i].a = 0;

        }

        if ((i >= (510 + step)) && (i < (765 + step))) {
            colors[i].r = 0;
            colors[i].g = 0;
            colors[i].b = b;
            b++;
            colors[i].a = 0;

        }
        if ((i >= (765 + step)) && (i < (1020 + step))) {
            colors[i].r = 0;
            colors[i].g = 0;
            colors[i].b = 0;
            colors[i].a = a;
            a++;

        }
        //printf("c:%d (%d,%d,%d,%d)\n",i, colors[i].r, colors[i].g, colors[i].b,  colors[i].a);
        //getchar();
    }
}

void VMI::setColors2(Color *colors, int numColors, int begin, int end) {
    int i, 
        channel = 0,
        step = (numColors < MAX_NUM_COLORS) ? (MAX_NUM_COLORS / numColors) : 1;
    GLubyte r = 1;

    if (end > numColors) return;

    for (i = begin; i < end; i++) {
        
        if (channel == 0) {
            colors[i].r = r;
            colors[i].g = 0;
            colors[i].b = 0;
            colors[i].a = 0;
        }
        if (channel == 1) {
            colors[i].r = 0;
            colors[i].g = r;
            colors[i].b = 0;
            colors[i].a = 0;
        }
        if (channel == 2) {
            colors[i].r = 0;
            colors[i].g = 0;
            colors[i].b = r;
            colors[i].a = 0;
        }
        if (channel == 3) {
            colors[i].r = 0;
            colors[i].g = 0;
            colors[i].b = 0;
            colors[i].a = r;
        }
        
        channel++;

        if (channel > 3) {
            if (r == 255) r = 1;
            else r += step;
            channel = 0;
        }
        //printf("c:%d (%d,%d,%d,%d)\n", i, colors[i].r, colors[i].g, colors[i].b,  colors[i].a);
        //getchar();
    }
}

void VMI::setColors3(Color *colors, int numColors, GLubyte color) {
    int i;
    
    for (i = 0; i < numColors; i++) {

        colors[i].r = color;
        colors[i].g = 0;
        colors[i].b = 0;
        colors[i].a = 0;

        //printf("c:%d (%d,%d,%d,%d)\n", i, colors[i].r, colors[i].g, colors[i].b,  colors[i].a);
        //getchar();
    }
}

void VMI::setColors4(Color *colors, int numColors) {
    int i, c = 1;
    
    for (i = 0; i < numColors; i++) {

        colors[i].r = c & 0xFF;
        colors[i].g = (c & 0xFF00)     >> 8;
        colors[i].b = (c & 0xFF0000)   >> 16;
        colors[i].a = (c & 0xFF000000) >> 24;

        //printf("c:%d (%d,%d,%d,%d)\n", i, colors[i].r, colors[i].g, colors[i].b,  colors[i].a);
        c++; 
    }
    //getchar();
}

void VMI::viewColors(Color *colors, int numColors) {
    int i;

    printf("\n");
    for (i=0; i<numColors; i++) 
        printf("c:%d (%d,%d,%d,%d)\n", i, colors[i].r, colors[i].g, colors[i].b,  colors[i].a);
}
