#include <iostream>
#include <getopt.h>
#include <fstream>
#include <sstream>
#include <pthread.h>
#define GL_GLEXT_PROTOTYPES 1
#define GL3_PROTOTYPES 1

#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/freeglut_ext.h>


#include "aruco.h"

#include "GeoMeshLoader.h"
#include "GeoLodTreeLibrary.h"
#include "VertexData.h"

using namespace cv;
using namespace aruco;

void unitQuad(float size);

Geometry::GeoMeshLoader meshloader;
Geometry::Mesh *themesh;
Geometry::LodTreeLibrary* tree;

string TheInputVideo;
string TheIntrinsicFile;
bool The3DInfoAvailable=false;
bool isIntrinsicFileYAML=false;
int threshold1=7, threshold2=1;
float TheMarkerSize=-1;
MarkerDetector PPDetector;
VideoCapture TheVideoCapturer;
vector<Marker> TheMarkers;
Mat TheInputImage,TheUndInputImage,TheResizedImage;
CameraParameters TheCameraParams;
Size TheGlWindowSize;
bool TheCaptureFlag=true;
bool readIntrinsicFile(string TheIntrinsicFile,Mat & TheIntriscCameraMatrix,Mat &TheDistorsionCameraParams,Size size);
void readArguments ( int argc,char **argv );
void usage();
void vDrawScene();
void vIdle();
void vResize( GLsizei iWidth, GLsizei iHeight );
void vMouse(int b,int s,int x,int y);
GLuint GoghT[3]; //two Van Gogh + one Teapot texture RGB
GLuint VideosT[3];
VideoCapture* video[3]={0,0,0};
Mat frame[3];
const int StatsSize=10000;
char systemstats[StatsSize];
/*GTGeometry*/
class myIndexData: public Geometry::IndexData {
private:
	Geometry::Mesh *m;
	GLuint *m_IndexBuffer;
	GLuint *m_VertexBuffer;
	float **vertices;
	unsigned int **indices;
	unsigned int *currentsizes;

	GLuint texturas[2];
	int rot;
	int offset;

	int* size;
	int* normalof;
	int* texcoorof;

public:

	myIndexData (Geometry::Mesh *mesh): Geometry::IndexData(), m(mesh)
	{
		rot=0;

		size=new int[mesh->mSubMeshCount];
		normalof=new int[mesh->mSubMeshCount];
		texcoorof=new int[mesh->mSubMeshCount];
		//http://www.opengl.org/wiki/VBO_-_just_examples
		m_IndexBuffer=new GLuint[mesh->mSubMeshCount];
		m_VertexBuffer=new GLuint[mesh->mSubMeshCount];
		glGenBuffers (mesh->mSubMeshCount, m_IndexBuffer);
		glGenBuffers (mesh->mSubMeshCount, m_VertexBuffer);
		indices=new unsigned int*[mesh->mSubMeshCount];
		vertices=new float*[mesh->mSubMeshCount];
		currentsizes=new unsigned int[mesh->mSubMeshCount];


		for (int i=0;i<mesh->mSubMeshCount;i++) {
			indices[i]=new unsigned int[mesh->mSubMesh[i].mIndexCount];
			//rgh: modified later in SetIndex calls
			//	for (int j=0;j<mesh->mSubMesh[i].mIndexCount;j++)
			//		indices[i][j]=mesh->mSubMesh[i].mIndex[j];

			glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, m_IndexBuffer[i]);
			glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(int)*mesh->mSubMesh[i].mIndexCount, indices[i], GL_DYNAMIC_DRAW);


			/*} rgh: do a vertexbuffer for each submesh*/
			Geometry::VertexBuffer *v=mesh->mSubMesh[i].mVertexBuffer;
			size[i]=0;
			normalof[i]=0;
			texcoorof[i]=0;
			if (v->mVertexInfo && Geometry::VERTEX_POSITION) {size[i] +=3; normalof[i]+=3; texcoorof[i]+=3;}
			if (v->mVertexInfo && Geometry::VERTEX_NORMAL) {size[i] +=3; texcoorof[i]+=3;}
			if (v->mVertexInfo && Geometry::VERTEX_TEXCOORDS) size[i] +=2;
			vertices[i]=new float[v->mVertexCount*size[i]];

			for (int k=0;k<v->mVertexCount;k++) {
				if (v->mVertexInfo && Geometry::VERTEX_POSITION)
					for (int j=0;j<3;j++) {
						vertices[i][k*size[i]+j]=v->mPosition[k][j];
					}
					if (v->mVertexInfo && Geometry::VERTEX_NORMAL) 
						for (int j=0;j<3;j++) {
							vertices[i][k*size[i]+normalof[i]+j]=v->mNormal[k][j];
						}
						if (v->mVertexInfo && Geometry::VERTEX_TEXCOORDS)
							for (int j=0;j<2;j++) {
								vertices[i][k*size[i]+texcoorof[i]+j]=v->mTexCoords[k][j];
							}
			}


			glBindBuffer (GL_ARRAY_BUFFER, m_VertexBuffer[i]);
			glBufferData(GL_ARRAY_BUFFER, sizeof(float)*size[i]*v->mVertexCount, vertices[i], GL_STATIC_DRAW);

			glEnableClientState(GL_VERTEX_ARRAY);
			glVertexPointer(3, GL_FLOAT, sizeof(float)*size[i], (char*)0);   //The starting point of the VBO, for the vertices
			glEnableClientState(GL_NORMAL_ARRAY);
			glNormalPointer(GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*normalof[i]);   //The starting point of normals,
			glClientActiveTexture(GL_TEXTURE0);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glTexCoordPointer(2, GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*texcoorof[i]);   //The starting point of texcoords,
		}
		//load textures

		IplImage* texts[2] = {0,0}; 
		texts[0]=cvLoadImage("ml06lef.png");//leaf
		texts[1]=cvLoadImage("ML15brk.tif");//trunc

		int w=texts[0]->width, h=texts[0]->height;

		unsigned char* hoja=new unsigned char[w*h*4];
		for (int i=0; i<h;i++)
			for (int j=0;j<w;j++) {
				unsigned int t=0;
				for (int k=0;k<3;k++) {
					t+=hoja[i*w*4 + j*4 + k]=texts[0]->imageData[i*w*3 + j*3 +k];
				}
				hoja[i*w*4 + j*4 +3]= t==0?0:0xff;
			}

			if (!texts[0]||!texts[1])
			{
				printf ("Error loading tree textures\n");
				exit (-1);
			}


			assert (texts[0]->nChannels==3 && texts[1]->nChannels==3 &&
				!strcmp(texts[0]->channelSeq, "BGR") &&
				!strcmp(texts[1]->channelSeq, "BGR") );

			glGenTextures( 2, texturas );
			for (int i=0;i<2;i++) {
				//http://www.nullterminator.net/gltexture.html
				glBindTexture( GL_TEXTURE_2D, texturas[i]);
				glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
					GL_LINEAR_MIPMAP_NEAREST );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
				glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
				if (i==1) gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, texts[i]->width, texts[i]->height,
					GL_BGR, GL_UNSIGNED_BYTE, texts[i]->imageData	 );
				else
					gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, texts[i]->width, texts[i]->height,
					GL_BGRA, GL_UNSIGNED_BYTE, hoja	 );
			}


	}
	~myIndexData (){}
	int currentmesh;
	void Begin (unsigned int submeshid, unsigned int indexcount)
	{
		//printf ("Begin: %d, %d\n", submeshid, indexcount);
		currentmesh=submeshid;
		currentsizes[submeshid]=indexcount;
		glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, m_IndexBuffer[submeshid]);
	}
	void SetIndex (unsigned int i, unsigned int index) {
		//printf ("SetIndex: %d, %d\n", submeshid, index);
		indices[currentmesh][i]=index;
	}
	void End()
	{
		glBufferSubData(GL_ELEMENT_ARRAY_BUFFER, 0, currentsizes[currentmesh]*sizeof(int), indices[currentmesh]);
		//printf ("End\n");
	}
	void BorrowIndexData(const Geometry::IndexData* indexdata) {printf ("Error, BorrowIndexData called unexpectedly\n"); exit (-4);}

	void Render(float s) {

		glPolygonMode (	GL_FRONT_AND_BACK, GL_FILL);
		float sf=m->mMeshBounds.scaleFactor/s/2;
		glTranslatef(0,s/3,0);
		glScalef(sf, sf, sf);
		float d=m->mMeshBounds.maxY-m->mMeshBounds.minY;
		glEnable(GL_BLEND);
		glDisable ( GL_ALPHA_TEST ) ;
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		//	glDisable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);


		float c[4]={1,1,1,1};
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, c);
		glColor4f (1,1,1,1);
		//glRotatef (rot,0 ,1,0);
		int current=0;
		int i;
		glBindTexture (GL_TEXTURE_2D, texturas[1]);
		for (i=0;i<m->mSubMeshCount;i++) {
			if (tree->GetLeavesSubMesh()==i) continue;
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_IndexBuffer[i]);
			glBindBuffer (GL_ARRAY_BUFFER, m_VertexBuffer[i]);

			glEnableClientState(GL_VERTEX_ARRAY);
			glVertexPointer(3, GL_FLOAT, sizeof(float)*size[i], (char*)0);   //The starting point of the VBO, for the vertices
			if (normalof[i]!=texcoorof[i]) {
				glEnableClientState(GL_NORMAL_ARRAY);
				glNormalPointer(GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*3);   //The starting point of normals,
			}
			if (texcoorof[i]!=size[i]) {
				glClientActiveTexture(GL_TEXTURE0);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
				glTexCoordPointer(2, GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*6);   //The starting point of texcoords,
			}
			glDrawElements(GL_TRIANGLE_STRIP, currentsizes[i], GL_UNSIGNED_INT, (char*)0);
		}
		i=tree->GetLeavesSubMesh();
		glBindTexture (GL_TEXTURE_2D, texturas[0]);

		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		glAlphaFunc ( GL_GREATER, 0.5 ) ;
		glEnable(GL_BLEND);
		//glEnable ( GL_ALPHA_TEST ) ;
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);

		/*rgh: uno a uno, como hace GeoTool
		glBegin(GL_TRIANGLES	);
		for (int j=0;j<currentsizes[i];j++) {
		int k=indices[i][j];
		glNormal3fv(vertices[i]+8*k+3);
		glTexCoord2fv(vertices[i]+8*k+6);
		glVertex3fv(vertices[i]+8*k);
		}
		glEnd();*/
		/*rgh: Ni siquiera la gente de Castellón pudo hacer funcionar esto (mirar código GeoTool). Dejo de calentarme la cabeza. 
		Edit: Lo he echado a andar :o)
		*/
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_IndexBuffer[i]);
		glBindBuffer (GL_ARRAY_BUFFER, m_VertexBuffer[i]);

		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(3, GL_FLOAT, sizeof(float)*size[i], (char*)0);   //The starting point of the VBO, for the vertices 
		if (normalof[i]!=texcoorof[i]) {
			glEnableClientState(GL_NORMAL_ARRAY);
			glNormalPointer(GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*normalof[i]);   //The starting point of normals,
		}
		if (texcoorof[i]!=size[i]) {
			glClientActiveTexture(GL_TEXTURE0);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glTexCoordPointer(2, GL_FLOAT, sizeof(float)*size[i], (char*)0+sizeof(float)*texcoorof[i]);   //The starting point of texcoords,
		}
		glDrawElements(GL_TRIANGLES, currentsizes[i], GL_UNSIGNED_INT, (char*)0);


	}

};



myIndexData *indexdata;

const	int uM=10;
double modelview_matrix[uM][16];
int undetected[uM];

void* updateVideoTexture(void *thread_id){
	int t=*(int*)thread_id;
	//printf ("update video texture called %d\n", t);
	while (video[t]==0) {
		sleep (1);
	}
	//printf ("update video texture called; after while\n");
	double frames=video[t]->get(CV_CAP_PROP_FRAME_COUNT);
	double fps=video[t]->get(CV_CAP_PROP_FPS);
	while (true) {
		/*if (undetected[t]>10) {
		usleep(1000*1000);
		printf ("undetected[%d]=%d\n", t, undetected[t]);
		continue;
		}*/
		//printf ("undetected[%d]=%d\n", t, undetected[t]);
		double pos;
		pos=video[t]->get(CV_CAP_PROP_POS_FRAMES);
		if (pos==frames-1) 
			video[t]->set(CV_CAP_PROP_POS_FRAMES, 0);
		*(video[t])>>frame[t];	
		usleep(1000*1000/fps);
	}
	pthread_exit(0);
	//return 0;
}

void* updateSystemStats (void *thread_id){
	while (true) {
		system ("top -b -n 1 |head -n 15> stats.txt");
		FILE *f=fopen ("stats.txt","rb");
		int read=fread (systemstats, 1, StatsSize, f);
		systemstats[read]='\0';
		usleep (1000*500);
	}
	pthread_exit(0);
}



void init() {
	//GTGeometry
	themesh=meshloader.load ("ML06a_01.mesh");
	/* Crea una nueva ventana e interfiere con ésta. Las texturas se pierden, y el árbol de castellón explota.
	Ogre::String lConfigFileName = "";
	Ogre::String lPluginsFileName = "";
	Ogre::String lLogFileName = "ogre.log";
	Ogre::Root * lRoot=new Ogre::Root();//lConfigFileName, lPluginsFileName, lLogFileName);

	Ogre::SceneManager *mSceneMgr=lRoot->createSceneManager("DefaultSceneManager");
	Ogre::ResourceGroupManager::getSingleton().addResourceLocation(".", "FileSystem", "data", true);
	lRoot->showConfigDialog();
	Ogre::RenderWindow* window = lRoot->initialise(true);//do not create window
	//Ogre::ResourceGroupManager::getSingleton().createResourceGroup(Ogre::String("Arbol"));
	Ogre::Entity *entity = mSceneMgr->createEntity("Arbol",Ogre::String("ML06a_01.mesh"));          
	//Ogre::MeshPtr ogremesh=Ogre::MeshManager::getSingleton().load(Ogre::String("ML06a_01.mesh"), Ogre::String("Arbol"));
	*/
	indexdata=new myIndexData (themesh);
	if (!meshloader.GetLodStripsData() || !meshloader.GetTreeSimpSeq())
	{
		printf ("Tree mesh has no LOD\n");
		exit (-3);	
	}

	tree=new Geometry::LodTreeLibrary(meshloader.GetLodStripsData(), meshloader.GetTreeSimpSeq(),themesh, indexdata);

	//
	IplImage* Gogh[3] = {0,0,0}; 

	video[0]=new VideoCapture("UJI_treeshadown.avi");
	video[2]=new VideoCapture("3-skull.avi");
	video[1]=new VideoCapture("Leyendas.avi");
	//create textures, etc
	Gogh[0]=cvLoadImage("Gogh1.jpg");
	Gogh[1]=cvLoadImage("Gogh2.jpg");
	Gogh[2]=cvLoadImage("RGB.jpg");


	if (!Gogh[0]||!Gogh[1])
	{
		printf ("Error loading Van Gogh\n");
		exit (-1);
	}


	assert (Gogh[0]->nChannels==3 && Gogh[1]->nChannels==3 &&
		!strcmp(Gogh[0]->channelSeq, "BGR") &&
		!strcmp(Gogh[1]->channelSeq, "BGR") );

	glGenTextures( 3, GoghT );
	for (int i=0;i<3;i++) {
		//http://www.nullterminator.net/gltexture.html
		glBindTexture( GL_TEXTURE_2D, GoghT[i] );
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
			GL_LINEAR_MIPMAP_NEAREST );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
		gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, Gogh[i]->width, Gogh[i]->height,
			GL_BGR, GL_UNSIGNED_BYTE, Gogh[i]->imageData	 );
	}

	glGenTextures(3, VideosT);
	for (int i=0;i<3;i++) {
		//http://www.nullterminator.net/gltexture.html
		glBindTexture( GL_TEXTURE_2D, VideosT[i] );
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
			GL_LINEAR_MIPMAP_NEAREST );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	}

	static pthread_t threads[4];
	static int t[3]={0,1,2};
	pthread_create (threads+0, 0,  updateVideoTexture, &t[0]);
	pthread_create (threads+1, 0,  updateVideoTexture, &t[1]);
	pthread_create (threads+2, 0,  updateVideoTexture, &t[2]);
	pthread_create (threads+3, 0, updateSystemStats, &t[0]);

	for (int m=0;m<uM;m++)
		undetected[m]=100;
}

void vkeyb(unsigned char key, int x, int y)
{
	if (key=='a')
		threshold1-=2;
	else if (key=='s')
		threshold2--;
	else if (key=='d')
		threshold1+=2;
	else if (key=='w')
		threshold2++;

	if (threshold1<3)
		threshold1=3;
	//printf ("t1=%d, t2=%d\n", threshold1, threshold2);

	float clf=tree->GetCurrentLodFactor();
	if (key=='g' )
		clf+=0.01;
	if (key=='h')
		clf-=0.01;

	//printf ("clf=%f\n", clf);
	//sleep (5);
	if (clf<=1 && clf >=0)	
		tree->GoToLod(clf);
}
/************************************
*
*
*
*
************************************/

int main(int argc,char **argv)
{
	try
	{
		if(argc==1) usage();
		readArguments (argc,argv);
		if (TheIntrinsicFile==""){cerr<<"-f option required"<<endl;return -1;}
		if (TheMarkerSize==-1){cerr<<"-s option required"<<endl;return -1;}
		//read from camera
		if (TheInputVideo=="") {
			TheVideoCapturer.open(0);
		}
		else TheVideoCapturer.open(TheInputVideo);
		if (!TheVideoCapturer.isOpened())
		{
			cerr<<"Could not open video"<<endl;
			return -1;

		}

		//read first image
		TheVideoCapturer>>TheInputImage;
		//read camera paramters if passed
		if (isIntrinsicFileYAML)
			TheCameraParams.readFromXMLFile(TheIntrinsicFile);
		else
			TheCameraParams.readFromFile(TheIntrinsicFile);
		TheCameraParams.resize(TheInputImage.size());
		glutInit(&argc, argv);
		glutInitWindowPosition( 0, 0);
		glutInitWindowSize(TheInputImage.size().width,TheInputImage.size().height);
		glutInitDisplayMode( GLUT_RGB | GLUT_ALPHA | GLUT_DEPTH | GLUT_DOUBLE );
		glutCreateWindow( "Gilab Augmented Reality Test" );
		glutDisplayFunc( vDrawScene );
		glutIdleFunc( vIdle );
		glutReshapeFunc( vResize );
		glutMouseFunc(vMouse);
		glutKeyboardFunc (vkeyb);
		glClearColor( 0.0, 0.0, 0.0, 0.0 );
		glClearDepth( 1.0 );
		TheGlWindowSize=TheInputImage.size();
		vResize(TheGlWindowSize.width,TheGlWindowSize.height);
		init();
		glutMainLoop();

	}catch(std::exception &ex)

	{
		cout<<"Exception :"<<ex.what()<<endl;
	}
	pthread_exit(0);
}

void RenderTree (float size) {
	//http://www.gamedev.net/topic/466568-opengl-index-buffers--ibo-/
	//http://www.opengl.org/wiki/VBO_-_just_examples
	indexdata->Render(size);
}
/************************************
*
*
*
*
************************************/
void unitQuad(float size) {
	glBegin(GL_QUADS);
	glTexCoord2d(0,0);
	glVertex3f(0,0,0);
	glTexCoord2d(0,1);
	glVertex3f(0,0,size);
	glTexCoord2d(1,1);
	glVertex3f(size,0,size);
	glTexCoord2d(1,0);	
	glVertex3f(size,0,0);
	glEnd();
}
void vMouse(int b,int s,int x,int y)
{
	if (b==GLUT_LEFT_BUTTON && s==GLUT_DOWN) {
		TheCaptureFlag=!TheCaptureFlag;
	}

}

/************************************
*
*
*
*
************************************/
void axis(float size)
{
	glColor3f (1,0,0 );
	glBegin(GL_LINES);
	glVertex3f(0.0f, 0.0f, 0.0f); // origin of the line
	glVertex3f(size,0.0f, 0.0f); // ending point of the line
	glEnd( );


	glColor3f ( 0,1,0 );
	glBegin(GL_LINES);
	glVertex3f(0.0f, 0.0f, 0.0f); // origin of the line
	glVertex3f( 0.0f,size, 0.0f); // ending point of the line
	glEnd( );


	glColor3f (0,0,1 );
	glBegin(GL_LINES);
	glVertex3f(0.0f, 0.0f, 0.0f); // origin of the line
	glVertex3f(0.0f, 0.0f, size); // ending point of the line
	glEnd( );


}

void setColor(const bool light, const float *c){
	if (light) {
		glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, c);
	}
	else
		glColor3fv(c);
}

void myVideo(int v, float size, bool light) {
	glEnable(GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, VideosT[v]);
	glDisable(GL_LIGHTING);
	glColor3f(1,1,1);
	glPushMatrix();
	glTranslatef(-size/2, 0, -size/2);
	unitQuad(size);
	glPopMatrix();
}

void VanGogh (float size, float dist, float trans, bool light) {
	//two planes with Van Gogh images on them
	glEnable(GL_TEXTURE_2D);	
	const float c0[4]={1,1,1,1};
	glColor4f(1,1,1,1);
	glBindTexture( GL_TEXTURE_2D, GoghT[0]);
	glDisable(GL_LIGHTING);
	setColor(light, c0);
	glPushMatrix();
	glTranslatef(-size/2, 0, -size/2);
	unitQuad(size);
	glPopMatrix();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1,1,1,trans);
	glTranslatef(-size/2, size/2*dist, -size/2);
	glBindTexture( GL_TEXTURE_2D, GoghT[1] );
	unitQuad(size);
};
void Gilab (float size, bool light) {
	const float c0[4]={1,1,1,1};
	const float c1[4]={0,0.57,1,1};
	const float c2[4]={1,0.67,0,1};
	const float c3[4]={0.8,1,0.31,1};
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE);

	glColor4f(1,1,1,1);
	glDisableClientState(GL_VERTEX_ARRAY);

	glDisableClientState(GL_NORMAL_ARRAY);

	//glClientActiveTexture(GL_TEXTURE0);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	//light=false;
	if (light) {//rgh:test
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
	}
	else {
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);
	}
	glPushMatrix();
	glTranslatef(-size, size/6, -size);
	/*setColor(light, c0);
	glBegin(GL_QUADS);
	glVertex3f(0,0,0);
	glVertex3f(0,0,size*2);
	glVertex3f(size*2,0,size*2);
	glVertex3f(size*2,0,0);
	glEnd();*/
	glTranslatef(size/2,0,size/2);
	setColor(light, c1);
	glTranslatef(size/6,0, size/6);
	glutSolidCube(size/3);
	glTranslatef(size/3,0, 0);
	glutSolidCube(size/3);

	setColor(light, c2);
	glTranslatef(size/3,0, 0);
	glutSolidCube(size/3.5);

	setColor(light,c3); //central
	glTranslatef(-size/3,0, size/3);
	glutSolidCube(size/3.5);

	setColor(light, c1);
	glTranslatef(size/3,0,0);
	glutSolidCube(size/3);
	glTranslatef(0,0, size/3);
	glutSolidCube(size/3);

	glTranslatef(-size/3,0, 0);
	glutSolidCube(size/3);
	glTranslatef(-size/3,0, 0);
	glutSolidCube(size/3);

	glTranslatef(0,0, -size/3);
	glutSolidCube(size/3);

	glPopMatrix();
}

void GLSystemStats(float TheMarkerSize, float currentTime, char *text)
{
	void * const font=GLUT_STROKE_MONO_ROMAN;
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	const float fs=0.05;
	glColor3f(1,0,0);
	const char *c;
	//const char *text="Center of the Marker, 3D font";
	GLfloat h,w;
	h=glutStrokeHeight(font);
	//w=glutStrokeLength(font, (const unsigned char *)text);
	glPushMatrix();
	glRotatef(-90,1,0,0	);
	glTranslatef(-TheMarkerSize/2,TheMarkerSize/2,0);
	glScalef(fs*TheMarkerSize/h, fs*TheMarkerSize/h, -1);
	int lines=1;
	for (c=text; *c != '\0'; c++) { 
		glutStrokeCharacter(font , *c);
		if (*c=='\n') {
			glPopMatrix();
			glPushMatrix();
			glRotatef(-90,1,0,0);
			glTranslatef(-TheMarkerSize/2,TheMarkerSize/2-TheMarkerSize*lines*fs,0	);
			glScalef(fs*TheMarkerSize/h, fs*TheMarkerSize/h, -1);
			lines++;
		}
	}
	char fps[100];
	sprintf (fps, "FPS: %f\n", currentTime);
	glColor3f(0,0,1);
	for (c=fps; *c != '\0'; c++)
		glutStrokeCharacter (font, *c);
	glPopMatrix();


	glColor3f(0,1,1);
	glRasterPos3f(-TheMarkerSize/2,0,+TheMarkerSize/2);
	for (c="Corner of the Marker, Bitmap font"; *c != '\0'; c++) 
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10 , *c);

}

/************************************
*
*
*
*
************************************/
void vDrawScene()
{
	static clock_t clk=clock();
	clock_t tmp=clock();
	float current=(tmp-clk);
	clk=tmp;
	if (current!=0)
		current=CLOCKS_PER_SEC/current;
	if (TheResizedImage.rows==0) //prevent from going on until the image is initialized
		return;

	///clear
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glEnable ( GL_ALPHA_TEST ) ;
	glClearDepth (1); glClearColor(0,0,0,0);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glDisable(GL_DEPTH_TEST);
	///draw image in the buffer
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, TheGlWindowSize.width, 0, TheGlWindowSize.height, -1.0, 1.0);
	glViewport(0, 0, TheGlWindowSize.width , TheGlWindowSize.height);
	//	glDisable(GL_TEXTURE_2D);
	glPixelZoom( 1, -1);
	glRasterPos3f( 0, TheGlWindowSize.height  - 0.5, -1.0 );
	glDrawPixels ( TheGlWindowSize.width , TheGlWindowSize.height , GL_RGB , GL_UNSIGNED_BYTE , TheResizedImage.ptr(0) );

	///Set the appropriate projection matrix so that rendering is done in a enrvironment
	//like the real camera (without distorsion)
	//rgh: for later, transparency works.
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//glEnable(GL_LIGHTING);
	//glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glClear( GL_DEPTH_BUFFER_BIT );
	//glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_PROJECTION);
	double proj_matrix[16];
	MarkerDetector::glGetProjectionMatrix(TheCameraParams,TheInputImage.size(),TheGlWindowSize,proj_matrix,0.05,10);
	glLoadIdentity();
	//test:
	//glColor3f (1,0,1);
	//glScalef(1,1,1);
	//RenderTree (0.01);
	//glBegin(GL_QUADS); glVertex3f(-0.5,-0.5,0);glVertex3f(-0.5,0.5,0);glVertex3f(.5,.5,0);glVertex3f(.5,-0.5,0);glEnd();
	//glutSwapBuffers(); return;

	glLoadMatrixd(proj_matrix);



	//now, for each marker,

	float c[3]={0.5, 0.7, 0.1};
	float c2[3]={0.9, 0.2, 0.3};
	float c3[3]={0.2, 1, 0.5};
	static float dist=0.5, trans=0.5, scale=1; //distance and transparency of VanGoghPlane	
	static cv::Mat tvec, rvec;
	//	printf ("TheMarkers.size=%d\n", TheMarkers.size());
	for(unsigned int m=0;m<TheMarkers.size();m++)
	{
		switch (TheMarkers[m].id) {
		case 35:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[0]);
			undetected[0]=0;
			break;
		case 202:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[1]);
			undetected[1]=0;
			break;
		case 200:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[2]);	
			undetected[2]=0;
			break;
		case 491:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[3]);
			undetected[3]=0;
			break;
		case 123:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[4]);
			undetected[4]=0;
			break;
		case 625:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[5]);
			undetected[5]=0;
			break;
		case 201:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[6]);
			undetected[6]=0;
			break;
		case 203:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[7]);
			undetected[7]=0;
			tvec=TheMarkers[m].Tvec;
			rvec=TheMarkers[m].Rvec;
			break;
		case 204:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[8]);
			undetected[8]=0;
			break;
		case 364:
			TheMarkers[m].glGetModelViewMatrix(modelview_matrix[9]);
			undetected[9]=0;
			break;
		default:
			printf ("Id=%d\n", TheMarkers[m].id);
		}
	}

	for (int m=0;m<uM;m++) {
		undetected[m]++;
		if (undetected[m]<10) {
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
			glLoadMatrixd(modelview_matrix[m]);
			switch (m) {
			case 0:  //"UJI_treeshadown.avi" -> LodTrees Castellón
				//glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, c2);
				myVideo(0, TheMarkerSize, true); /*
												 glBindTexture(GL_TEXTURE_2D, GoghT[2]);
												 glRotatef(90,1,0,0);
												 glutSolidTeapot (TheMarkerSize/2);*/
				break;
			case 1: //"3-skull.avi" -> Raytracing Budapest
				myVideo(1, TheMarkerSize, true);
				break;
			case 2:
				myVideo(2, TheMarkerSize, true);
				break;
			case 3:
				glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, c);
				glutSolidSphere (TheMarkerSize, 20, 20);
				break;
			case 4: 
				//glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, c3);
				glScalef(scale,1,scale);
				VanGogh (TheMarkerSize, dist, trans, true);
				/*glRotatef(90,1,0,0);
				glutSolidTeapot (TheMarkerSize/1.5);*/
				break;
			case 5: 
				Gilab(TheMarkerSize, true);
				break;
			case 6:
				GLSystemStats(TheMarkerSize, current, systemstats);
				break;
			case 7: // control Van Gogh

				//for (int i=0;i<3;i++)
				//	printf ("%f ", rvec.at<float>(i,0));
				//printf ("%f ", tvec.at<float>(0,0)/tvec.at<float>(2,0));
				//printf ("\n");
				dist=(tvec.at<float>(0,0)/tvec.at<float>(2,0)+0.5)*2;
				trans=(tvec.at<float>(1,0)/tvec.at<float>(2,0)+0.5)*2;
				scale=3*(rvec.at<float>(2,0)+M_PI)/M_PI;	
				glColor3f(0,1,0);
				glDisable(GL_TEXTURE_2D);
				glDisable(GL_LIGHTING);
				glPushMatrix();
				glTranslatef(-TheMarkerSize/2, 0, -TheMarkerSize/2);
				unitQuad(TheMarkerSize);
				glPopMatrix();
				glTranslatef(0,TheMarkerSize/20,TheMarkerSize/20);
				char text[100];
				sprintf(text, "Van Gogh Control\nX=distance->%f\nY=tranparency->%f\nZ-Rotation=scale->%f\n", dist, trans, scale);
				GLSystemStats(TheMarkerSize, current, text);
				break;
			case 8: //LODTREE
				RenderTree(TheMarkerSize);
				break;
			case 9:
				Gilab(TheMarkerSize, true);
				//printf ("9\n");
				break;

			}
		}
	}


	glutSwapBuffers();

}


/************************************
*
*
*
*
************************************/
void vIdle()
{
	if(TheCaptureFlag){
		//capture image
		TheVideoCapturer.grab();
		TheVideoCapturer.retrieve( TheInputImage);
		TheUndInputImage.create(TheInputImage.size(),CV_8UC3);
		//transform color that by default is BGR to RGB because windows systems do not allow reading BGR images with opengl properly
		//rgh: test supposing non-distorting camera
		cv::cvtColor(TheInputImage,TheUndInputImage,CV_BGR2RGB);
		//	cv::cvtColor(TheInputImage,TheInputImage,CV_BGR2RGB);
		//remove distorion in image


		//	cv::undistort(TheInputImage,TheUndInputImage, TheCameraParams.CameraMatrix, TheCameraParams.Distorsion);
		PPDetector.setThresholdParams(threshold1, threshold2);
		//detect markers
		PPDetector.detect(TheUndInputImage,TheMarkers, TheCameraParams.CameraMatrix,Mat(),TheMarkerSize);
		//resize the image to th/e size of the GL window
		cv::resize(TheUndInputImage,TheResizedImage,TheGlWindowSize);

		//rgh: update video texture
		//updateVideoTexture(0);
		for (int i=0;i<3;i++) {
			glBindTexture( GL_TEXTURE_2D, VideosT[i] );
			gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, frame[i].cols, frame[i].rows,
				GL_BGR, GL_UNSIGNED_BYTE, frame[i].ptr()	 );
		}
	}
	glutPostRedisplay();
}


/************************************
*
*
*
*
************************************/
void vResize( GLsizei iWidth, GLsizei iHeight )
{
	TheGlWindowSize=Size(iWidth,iHeight);
	//not all sizes are allowed. OpenCv images have padding at the end of each line in these that are not aligned to 4 bytes
	if (iWidth*3%4!=0){
		iWidth+=iWidth*3%4;//resize to avoid padding
		vResize(iWidth,TheGlWindowSize.height);
	}
	else{
		//resize the image to the size of the GL window
		if (TheUndInputImage.rows!=0)
			cv::resize(TheUndInputImage,TheResizedImage,TheGlWindowSize);
	}
}


/************************************
*
*
*
*
************************************/
void usage()
{
	cout<<"This program test the ArUco Library \n\n";
	cout<<"-i <video.avi>: specifies a input video file. If not, images from camera are captures"<<endl;
	cout<<"-f <file>: if you have calibrated your camera, pass calibration information here so as to be able to get 3D marker info"<<endl;
	cout<<"-y <file.yml>: if you have calibrated your camera in yml format as provided by the calibration.cpp aplication in OpenCv >= 2.2"<<endl;
	cout<<"-s <size>: size of the marker's sides (expressed in meters!)"<<endl;

}


/************************************
*
*
*
*
************************************/
static const char short_options [] = "hi:f:s:y:";

static const struct option
	long_options [] =
{
	{ "help",           no_argument,   NULL,                 'h' },
	{ "input",     required_argument,   NULL,           'i' },
	{ "YAMLFile",     required_argument,   NULL,           'y' },
	{ "intFile",     required_argument,   NULL,           'f' },
	{ "size",     required_argument,   NULL,           's' },

	{ 0, 0, 0, 0 }
};

/************************************
*
*
*
*
************************************/
void readArguments ( int argc,char **argv )
{
	for ( ;; )
	{
		int index;
		int c;
		c = getopt_long ( argc, argv,
			short_options, long_options,
			&index );

		if ( -1 == c )
			break;
		switch ( c )
		{
		case 0:
			break;
		case 'h':
			usage ();
			exit ( EXIT_SUCCESS );
			break;
		case 'i':
			TheInputVideo=optarg;
			break;
		case 'f':
			TheIntrinsicFile=optarg;				
			isIntrinsicFileYAML=false;
			break;
		case 'y':
			TheIntrinsicFile=optarg;
			isIntrinsicFileYAML=true;
			break;
		case 's':
			TheMarkerSize=atof(optarg);
			break;
		default:
			usage ();
			exit ( EXIT_FAILURE );
		};
	}
}
